import shutil
import os
from glob import glob

from optparse import OptionParser

p = OptionParser(usage='%prog [OPTION...] [FILE...]',
                 description='Copy FILEs to job directories.')
p.add_option('--sub', metavar='FILE',
             help='prepare submit script with this job file')
p.add_option('--nodes', default='4:ppn=4:opteron',
             help='nodes on which to run')
p.add_option('--walltime', default='13:0:0',
             help='walltime for jobs')
p.add_option('--atoms', default='19-340',
             help='atom count slice.  Default: %default')
p.add_option('--copy', action='store_true',
             help='copy FILEs rather than symlinking')
p.add_option('--probe', action='store_true',
             help='no mass IO operations, but describe what would happen')
p.add_option('--remove', action='store_true',
             help='delete the given FILEs')
p.add_option('--seed', default='0',
             help='commaseparated seeds.  Default: %default')
p.add_option('--conditional', metavar='FILE',
             help='do nothing where FILE already exists')
p.add_option('--onlywhereexists', metavar='FILE',
             help='do something only where FILE exists')

opts, args = p.parse_args()

assert not (opts.remove and opts.copy)

fnames = args
if opts.sub:
    fnames.append(opts.sub)
print '%d source file%s' % (len(fnames), ['s', ''][len(fnames) == 1])

dirs = glob('O_Pt_*.?.?.??.???')
dirs.sort()

pwd = os.getcwd()

if opts.probe:
    operation = 'Probed'
if opts.remove:
    operation = 'Removed'
elif opts.copy:
    operation = 'Copied'
else:
    operation = 'Linked'

noperations = 0

nnotprobed = 0

print '%d target directories' % len(dirs)
submitdirs = []
for dir in dirs:
    for fname in fnames:
        dst = '%s/%s' % (dir, fname)
        #print dst
        #if opts.copy:
        #
        if opts.probe:
            if os.path.exists(dst):
                desc = 'file exists'
                noperations += 1
            else:
                desc = 'no such file'
                nnotprobed += 1
            print dst, desc
            continue
        
        if opts.conditional:
            if os.path.exists('%s/%s' % (dir, opts.conditional)):
                continue
        if opts.onlywhereexists:
            if not os.path.exists('%s/&s' % (dir, opts.onlywhereexists)):
                continue
        
        if opts.remove:
            try:
                os.remove(dst)
                noperations += 1
            except OSError:
                continue
        elif opts.copy:
            if not os.path.exists(dst):
                shutil.copyfile(fname, dst)
                noperations += 1
                submitdirs.append(dir)
        else:
            if not os.path.islink(dst):
                os.symlink('../%s' % fname, dst)
                noperations += 1
            submitdirs.append(dir)

if opts.probe:
    print 'Found %d files, missing %d' % (noperations, nnotprobed)
else:
    print '%s: %s' % (operation, noperations)

if opts.sub:
    fname = opts.sub
    id = fname
    if id.endswith('.py'):
        id = id[:-3]
    submitfname = 'multisub.%s' % id
    multisub = open(submitfname, 'w')
    print 'Jobs for submission: %d' % len(submitdirs)
    print 'Submit file: %s' % submitfname
    tokens = ['gpaw-qsub']
    
    tokens.append('-e stderr.%s.log' % id)
    tokens.append('-o stdout.%s.log' % id)
    for i, dir in enumerate(submitdirs):
        nodes = opts.nodes
        walltime = opts.walltime
        if nodes == 'auto':
            #nodes = '1:ppn=8'
            nodes = '1:ppn=4:opteron'
            walltime = '0:25:0'
            N = int(dir.split('.')[-1])
            if N > 160:
                walltime = '1:0:0'
            if N > 270:
                #nodes = '1:ppn=8'
                nodes = '2:ppn=4:opteron'
                walltime = '2:15:0'

        walltimetoken = '-l walltime=%s' % walltime
        nodetoken = '-l nodes=%s' % nodes
        
        jobname = '-N %s/%s' % (dir, fname)
        if dir == 'seed.00/Au.138':
            jobname = '-N YannTison'
        cmd = ' '.join(tokens + [nodetoken,
                                 walltimetoken, jobname, fname])
        print >> multisub, 'echo %d && cd %s && %s && cd %s' % (i, dir, cmd,
                                                                pwd)
