from ase.io import read
from gpaw import GPAW, Mixer, PoissonSolver
from gpaw.utilities import h2gpts
from gpaw.mpi import world

if world.size == 16:
    N, M = 4, 4
elif world.size == 32:
    N, M = 8, 4
else:
    assert world.size % 2 == 0
    N, M = world.size // 2, 2

system = read('geom.traj')
system.center(vacuum=5.0, axis=2)

calc = GPAW(mode='lcao',
            basis='dzp',
            nbands=(-len(system) // 2),
            txt='gpaw.clean.txt',
            mixer=Mixer(0.03, 7, 100.0),
            kpts=(2, 2, 1),
            gpts=h2gpts(0.18, system.get_cell(), idiv=8),
            poissonsolver=PoissonSolver(relax='GS'),
            parallel=dict(sl_default=(N, M, 64)),
            xc='RPBE')

system.pop(-1)
system.set_calculator(calc)
try:
    system.get_potential_energy()
except ValueError:
    calc.set(parallel=dict(sl_default=(N // 2, M, 64)))
    system.get_potential_energy()

#calc.set(txt='gpaw.clean.txt')
#system.get_potential_energy()
